# Definition of the class structures in file imas.py
import imas
import numpy
import sys
import os

'''
This sample program will create a pulse file (shot 13, run 1) and will
put an example of equilibirium IDS using put_slice methods.
'''

# This routine reads an array of pfsystems IDSs in the database, filling
# some fields of the IDSS

TEST_DATABASE_NAME = 'test'


def put_ids():
    """Class Itm is the main class for the UAL.

    It contains a set of field classes, each corresponding to a IDS
    defined in the UAL The parameters passed to this creator define the
    shot and run number. The second pair of arguments defines the
    reference shot and run and is used when the a new database is
    created, as in this example.

    """

    shot = 13
    time = 1
    interp = 1

    imas_obj = imas.ids(13, 1, 13, 1)
    # Create a new instance of database
    imas_obj.create_env("fydev", "test", "3")

    if imas_obj.isConnected():
        print('Creation of data entry OK!')
    else:
        print('Creation of data entry FAILED!')
        sys.exit()

    number = 10

    # Allocate a first generic vector and its time base
    lentime_1 = 3
    vect1DDouble_1 = numpy.empty([lentime_1])
    time_1 = numpy.empty([lentime_1])

    for i in range(lentime_1):
        time_1[i] = i
        vect1DDouble_1[i] = i * 10

    print('========================================================')
    print(time_1)
    print(vect1DDouble_1)

    # Allocate a second generic vector and its time base
    lentime_2 = 4
    vect1DDouble_2 = numpy.empty([lentime_2])
    time_2 = numpy.empty([lentime_2])

    for i in range(lentime_2):
        time_2[i] = i
        vect1DDouble_2[i] = i * 11

    print('========================================================')
    print(time_2)
    print(vect1DDouble_2)

    vect2DDouble_1 = numpy.zeros([3, 3])
    for i in range(3):
        for j in range(3):
            vect2DDouble_1[i, j] = i * 100 + j

    print('========================================================')
    print(vect2DDouble_1)
    # Allocate a second generic vector and its time base
    lentime_2 = 4
    vect1DDouble_2 = numpy.empty([lentime_2])
    time_2 = numpy.empty([lentime_2])

    for i in range(lentime_2):
        time_2[i] = i
        vect1DDouble_2[i] = i * 11

    print('========================================================')
    print(time_2)
    print(vect1DDouble_2)

    vect2DDouble_1 = numpy.zeros([3, 3])
    for i in range(3):
        for j in range(3):
            vect2DDouble_1[i, j] = i * 100 + j

    print('========================================================')
    print(vect2DDouble_1)

    vect2DDouble_2 = vect2DDouble_1 + 10000
    '''
    print( '========================================================')
    print( vect3DDouble_2)
    '''
    imas_obj.equilibrium.ids_properties.comment = 'This is a test ids'

    # A sample int

    # Mandatory to define this property
    imas_obj.equilibrium.ids_properties.homogeneous_time = 1
    imas_obj.equilibrium.resize(1)
    imas_obj.equilibrium.time_slice[0].profiles_2d.resize(2)
    imas_obj.equilibrium.time_slice[0].profiles_2d[0].Mesh_type.name = 'Mesh TYPE 1A'
    imas_obj.equilibrium.time_slice[0].profiles_2d[1].Mesh_type.name = 'Mesh TYPE 2B'

    imas_obj.equilibrium.time.resize(1)
    imas_obj.equilibrium.time_slice[0].profiles_2d[0].r.resize(3, 3)
    imas_obj.equilibrium.time_slice[0].profiles_2d[1].r.resize(3, 3)

    print('Start Put, writing first slice')
    imas_obj.equilibrium.time_slice[0].profiles_2d[0].r[:, 0] = vect2DDouble_1[0, :]
    imas_obj.equilibrium.time_slice[0].profiles_2d[1].r[:, 0] = vect2DDouble_2[0, :]
    imas_obj.equilibrium.time[0] = time_1[0]
    imas_obj.equilibrium.put()
    print('Completed Put ')

    for i in range(lentime_1):
        print('========================================================')
        print('vect3DDouble_1[i,:,:]')
        print(vect2DDouble_1[i, :])
        print('========================================================')

        imas_obj.equilibrium.time_slice[0].profiles_2d[0].r[:, i] = vect2DDouble_1[i, :]
        print('========================================================')
        print('imas_obj.equilibrium.time_slice[0].profiles_2d[0].r')
        print(imas_obj.equilibrium.time_slice[0].profiles_2d[0].r[:, i])
        print('========================================================')
        imas_obj.equilibrium.time_slice[0].profiles_2d[1].r[:, i] = vect2DDouble_2[i, :]
        imas_obj.equilibrium.time[0] = time_1[i]
        print(('Writing slice={0}'.format(i)))
        imas_obj.equilibrium.putSlice()

    print('========================================================')
    print(imas_obj.equilibrium.time_slice[0].profiles_2d[0].r)

    '''
        print( '========================================================')
        print (imas_obj.equilibrium.time_slice[0].profiles_2d[1].r)
        '''
    imas_obj.close()


put_ids()



from ..core.Functor import Functor
from ..core.Expression import Expression
from ..utils.typing import NumericType, as_array
import typing
from .interpolate import interpolate
import numpy as np


def integral(func, *args, **kwargs):
    return func.integral(*args, **kwargs)


def find_roots(func, *args, **kwargs) -> typing.Generator[typing.Any, None, None]:
    yield from func.find_roots(*args, **kwargs)



import pathlib
from spdm.core.File import File
from spdm.utils.logger import logger

WORKSPACE = "/home/salmon/workspace"  # "/ssd01/salmon_work/workspace/"

DATA_PATH = pathlib.Path(f"{WORKSPACE}/fytok_data/data")

if __name__ == '__main__':

    with File("/home/salmon/workspace/gacode/neo/tools/input/profile_data/iterdb141459.03890", mode="r", format="iterdb") as fid:
        doc = fid.read()
        eq = doc.dump()

    logger.debug(eq)




from typing import List
from spdm.core.Query import Query
from spdm.core.Path import Path
from spdm.core.Entry import Entry, EntryCombine
from spdm.utils.tags import _not_found_
from spdm.utils.logger import logger
import pathlib
import sys
import unittest
import numpy as np


class TestEntryCombiner(unittest.TestCase):
    data = [
        {"id": 0,
            "value": 1.23,
            "c": "I'm {age}!",
            "d": {"e": "{name} is {age}", "f": "{address}", "g": [1, 2, 3]}},
        {"id": 1,
            "c": "I'm {age}!",
            "d": {"e": "{name} is {age}", "f": "{address}"}},
        {"id": 2,
            "value": 4.23,
            "c": "I'm {age}!",
            "d": {"e": "{name} is {age}", "f": "{address}", "g": [4, 5, 7]}}
    ]
    atoms = [
        {"label": "H", "z": 1, "element": [{"a": 1, "z_n": 1, "atoms_n": 1}], },
        {"label": "D", "z": 1, "element": [{"a": 2, "z_n": 1, "atoms_n": 1}], },
        {"label": "T", "z": 1, "element": [{"a": 3, "z_n": 1, "atoms_n": 1}], },
    ]
    models = [
        {
            "code": {"name": "dummy0"},
            "profiles_1d": {"ion": atoms}
        },
        {
            "code": {"name": "dummy1"},
            "profiles_1d": {
                "ion": [
                    {"label": "H", "density": np.random.random(128)},
                    {"label": "D", },
                    {"label": "T", "density":  np.random.random(128)},
                ]
            }
        },
        {
            "code": {"name": "dummy2"},
            "profiles_1d": {
                "ion": [
                    {"label": "D", "density": np.random.random(128)},
                    {"label": "T", "temperature": np.random.random(128)},
                    {"label": "H", "density": np.random.random(128)},
                ]
            }
        },

    ]

    # def test_get(self):
    #     d = EntryCombine(self.data)
    #     self.assertEqual(d.child("value").pull(), sum([v.get("value", 0.0) for v in self.data]))
    #     self.assertEqual(d.child("d.g").pull(), self.data[0]["d"]["g"]+self.data[2]["d"]["g"])

    def test_combine(self):
        models = [
        {
            "code": {"name": "dummy1"},
            "profiles_1d": {
                "ion": [
                    {"label": "H", "density": np.random.random(128)},
                    {"label": "D", },
                    {"label": "T", "density":  np.random.random(128)},
                ]
            }
        },
        {
            "code": {"name": "dummy2"},
            "profiles_1d": {
                "ion": [
                    {"label": "D", "density": np.random.random(128)},
                    {"label": "T", "temperature": np.random.random(128)},
                    {"label": "H", "density": np.random.random(128)},
                ]
            }
        },

    ]
        d = List(models)
        self.assertTrue(np.array_equal(
                 d["profiles_1d", "ion", Query(label="H"), "density"].combine.value,
                        models[1]["profiles_1d"]["ion"][0]["density"] +
                        models[2]["profiles_1d"]["ion"][2]["density"])
                        )

    # def test_cache(self):
    #     cache = {}
    #     d = EntryCombiner(self.data )
    #     expected = sum([d["value"] for d in self.data])

    #     c = d.extend("value")
    #     self.assertEqual(c.pull(), expected)
    #     self.assertEqual(cache["value"], expected)
    #     c.push(5)
    #     self.assertEqual(cache["value"], 5)

    #     d.extend("test_cache").push("just test cache")
    #     self.assertEqual(d.get("test_cache",default_value=_undefined_), _undefined_)
    #     self.assertEqual(d.get("test_cache"), cache["test_cache"])

# class TestEntryWrapper(unittest.TestCase):
#     data = [
#         {"id": 0,
#          "value": 1.23,
#          "c": "I'm {age}!",
#          "d": {"e": "{name} is {age}", "f": "{address}", "g": [1, 2, 3]}},
#         {"id": 1,
#          "value": 2.23,
#          "c": "I'm {age}!",
#          "d": {"e": "{name} is {age}", "f": "{address}"}},
#         {"id": 2,
#          "value": 4.23,
#          "c": "I'm {age}!",
#          "d": {"e": "{name} is {age}", "f": "{address}", "g": [4, 5, 7]}}
#     ]

#     def test_get(self):
#         entry = Entry(self.data)
#         d = EntryWrapper(entry)

#     def test_property(self):
#         entry = Entry(self.data)
#         d = EntryWrapper(entry)

#     def test_put(self):
#         entry = Entry(self.data)
#         d = EntryWrapper(entry)

#     def test_erase(self):
#         entry = Entry(self.data)
#         d = EntryWrapper(entry)

#     def test_cache(self):
#         entry = Entry(self.data)
#         d = EntryWrapper(entry)
#     #     d = Entry(cache)

#     #     d.extend("a").put("hello world {name}!")
#     #     self.assertEqual(cache["a"], "hello world {name}!")

#     #     d["c"][_next_] = 1.23455
#     #     d["c"][_next_] = {"a": "hello world", "b": 3.141567}

#     #     self.assertEqual(cache["c"][0],  1.23455)
#     # def test_append(self):
#     #     d = Entry()
#     #     d.extend(_next_).put({"a": 1, "b": 2})

#     #     self.assertEqual(d.count, 1)
#     #     # self.assertTrue(d.__category__ | Entry.Category.LIST)
#     #     self.assertEqual(d.extend([0, "a"]).get(), 1)
#     #     self.assertEqual(d.extend([0, "b"]).get(), 2)


if __name__ == '__main__':
    unittest.main()



import pprint
import sys

import matplotlib.pyplot as plt
import numpy as np
from scipy import constants  


if __name__ == "__main__":

    sys.path.append("/home/salmon/workspace/SpDev/SpDB")

    from spdm.core.File import File
    from ..util.logger import logger

    fp = File("/home/salmon/workspace/SpDev/SpDB/examples/data/FuYun/modules/physics/genray/template/g063982.04800",
              metadata={"$schema": "file/geqdsk"})

    logger.debug(fp)
